package dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus;

package .tmp;

import com.google.common.base.Preconditions;
import com.mojang.datafixers.kinds.App;
import com.mojang.datafixers.kinds.Applicative;
import com.mojang.datafixers.util.Function3;
import com.mojang.serialization.Codec;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import dev.shadowsoffire.apotheosis.Apotheosis;
import dev.shadowsoffire.apotheosis.adventure.affix.Affix;
import dev.shadowsoffire.apotheosis.adventure.affix.effect.DamageReductionAffix;
import dev.shadowsoffire.apotheosis.adventure.loot.LootRarity;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.GemClass;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus.GemBonus;
import dev.shadowsoffire.placebo.util.StepFunction;
import java.util.Map;
import java.util.Objects;
import java.util.function.Consumer;
import net.minecraft.ChatFormatting;
import net.minecraft.network.chat.Component;
import net.minecraft.tags.DamageTypeTags;
import net.minecraft.world.damagesource.DamageSource;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.item.ItemStack;

public class DamageReductionBonus extends GemBonus {
  protected final DamageReductionAffix.DamageType type;
  
  protected final Map<LootRarity, StepFunction> values;
  
  public static Codec<dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus.DamageReductionBonus> CODEC;
  
  static {
    CODEC = RecordCodecBuilder.create(inst -> inst.group(gemClass(), (App)DamageReductionAffix.DamageType.CODEC.fieldOf("damage_type").forGetter(()), (App)VALUES_CODEC.fieldOf("values").forGetter(())).apply((Applicative)inst, dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus.DamageReductionBonus::new));
  }
  
  public DamageReductionBonus(GemClass gemClass, DamageReductionAffix.DamageType type, Map<LootRarity, StepFunction> values) {
    super(Apotheosis.loc("damage_reduction"), gemClass);
    this.type = type;
    this.values = values;
  }
  
  public Component getSocketBonusTooltip(ItemStack gem, LootRarity rarity) {
    float level = ((StepFunction)this.values.get(rarity)).get(0.0F);
    return (Component)Component.m_237110_("affix.apotheosis:damage_reduction.desc", new Object[] { Component.m_237115_("misc.apotheosis." + this.type.getId()), Affix.fmt(100.0F * level) }).m_130940_(ChatFormatting.YELLOW);
  }
  
  public float onHurt(ItemStack gem, LootRarity rarity, DamageSource src, LivingEntity user, float amount) {
    if (!src.m_269533_(DamageTypeTags.f_268738_) && !src.m_269533_(DamageTypeTags.f_268413_) && this.type.test(src)) {
      float level = ((StepFunction)this.values.get(rarity)).get(0.0F);
      return amount * (1.0F - level);
    } 
    return super.onHurt(gem, rarity, src, user, amount);
  }
  
  public GemBonus validate() {
    Preconditions.checkNotNull(this.type, "Invalid DamageReductionBonus with null type");
    Preconditions.checkNotNull(this.values, "Invalid DamageReductionBonus with null values");
    Preconditions.checkArgument(this.values.entrySet().stream().mapMulti((entry, consumer) -> {
            consumer.accept(entry.getKey());
            consumer.accept(entry.getValue());
          }).allMatch(Objects::nonNull), "Invalid DamageReductionBonus with invalid values");
    return this;
  }
  
  public boolean supports(LootRarity rarity) {
    return this.values.containsKey(rarity);
  }
  
  public int getNumberOfUUIDs() {
    return 0;
  }
  
  public Codec<? extends GemBonus> getCodec() {
    return (Codec)CODEC;
  }
}
